library SWFContentNew;

uses
  SysUtils,
  Classes,
  Windows,
  contplug,
  SWFFileInfo;

{$E wdx}
{$R *.res}
{$R SWFIcon.res}

const
  FieldsNames: array [0..8] of PAnsiChar = ('MovieFlashVersion', 'Compressed',
    'FPS', 'FramesCount', 'Width', 'Height', 'Dimensions', 'TimeString', 'Time');
  FieldsTypes: array[0..High(FieldsNames)] of Integer = (1,6,1,1,1,1,8,3,5);

const
  DetectStr = 'EXT="SWF"&(SIZE>22)&(([0]="C"&[1]="W"&[2]="S")|([0]="F"&[1]="W"&[2]="S"))';

// function StrCopyW from jcl
function StrCopyW(Dest: PWideChar; const Source: PWideChar): PWideChar; inline; forward;

function StrCopyW(Dest: PWideChar; const Source: PWideChar): PWideChar;
var
  Src: PWideChar;
begin
  Result := Dest;
  if Dest <> nil then
  begin
    Src := Source;
    if Src <> nil then
      while Src^ <> #0 do
      begin
        Dest^ := Src^;
        Inc(Src);
        Inc(Dest);
      end;
    Dest^ := #0;
  end;
end;

function Hrs_Min_Sec(Secs: Extended): string;
const
  OneSec = 1/24/3600;
var
  Total: Extended;
begin
  Total  := Secs * OneSec;
  Result := FormatDateTime ('hh:nn:ss', Frac(Total));
end;

procedure ContentGetDetectString(DetectString: PAnsiChar; MaxLen: integer); stdcall;
begin
  StrLCopy(DetectString, PAnsiChar(DetectStr), MaxLen - 1);
end;

function ContentGetSupportedField(FieldIndex:integer;FieldName:pansichar;
  Units:pansichar;maxlen:integer):integer; stdcall;
begin
  if FieldIndex in [0..High(FieldsNames)] then
  begin
    StrCopy(Units,'');
    StrLCopy(FieldName,FieldsNames[FieldIndex],maxlen-1);
    Result := FieldsTypes[FieldIndex];
  end else Result := ftNoMoreFields;
end;

function ContentGetValueW(FileName: PWideChar; FieldIndex, UnitIndex: integer; FieldValue: PByte;
  MaxLen, Flags: integer): integer; stdcall;
var
  RetVal: Integer;
  SWF: TSWFHeader;
  ATime: ttimeformat;
  DT: TDateTime;
  _DummyWord: Word;
  AFileName: WideString;
  TimeDbl: double;
  ATimeStr: WideString;
  DimensionsStr: AnsiString;
begin
  AFileName := FileName;

  try
    if not GetSwfFileHeader(AFileName, SWF) then
    begin
      Result := ft_fieldempty;
      Exit;
    end;
  except
  end;

  case FieldIndex of
    0: RetVal := SWF.Version;
    1: RetVal := Integer(SWF.Signature = 'CWS');
    2: RetVal := SWF.FrameRate;
    3: RetVal := SWF.FrameCount;
    4: RetVal := SWF.FrameSize.Xmax div 20;
    5: RetVal := SWF.FrameSize.Ymax div 20;
    6: DimensionsStr := Format('%d x %d',
         [SWF.FrameSize.Xmax div 20, SWF.FrameSize.Ymax div 20]);
    7: begin
         if SWF.FrameRate = 0 then TimeDbl := 0
           else TimeDbl := SWF.FrameCount / SWF.FrameRate;
       end;
    8: begin
         if SWF.FrameRate = 0 then DT := 0
           else DT := (SWF.FrameCount/SWF.FrameRate) * (1/24/3600);
         DecodeTime(DT, ATime.wHour, ATime.wMinute, ATime.wSecond, _DummyWord);
       end;
  end; //case
  case FieldIndex of
    0..5: PInteger(FieldValue)^ := Integer(RetVal);
    6: StrLCopy(PAnsiChar(FieldValue), PAnsiChar(DimensionsStr), maxlen-1);
    7: begin
         if TimeDbl = 0 then ATimeStr := '00:00:00'
           else ATimeStr := Hrs_Min_Sec(TimeDbl);
         pdouble(FieldValue)^ := TimeDbl;
         StrCopyW(pwidechar(FieldValue)+sizeof(double) div 2, PWideChar(ATimeStr));
       end;
    8: ptimeformat(FieldValue)^ := ATime;
  end; //case
  if FieldIndex in [0..High(FieldsNames)] then
    Result := FieldsTypes[FieldIndex]
    else Result := ft_fieldempty;
end;

function ContentGetValue(FileName:pansichar;FieldIndex,UnitIndex:integer;FieldValue:pbyte;
  maxlen,flags:integer):integer; stdcall;
begin
  Result := ContentGetValueW(PWideChar(FileName), FieldIndex, UnitIndex, FieldValue, maxlen, flags);
end;

function ContentGetDefaultSortOrder(FieldIndex: integer): integer; stdcall;
begin
  case FieldIndex of
    1, 8:
      Result := 1;
  else
    Result := -1;
  end;
end;

exports
  ContentGetDetectString,
  ContentGetSupportedField,
  ContentGetValue,
  ContentGetValueW,
  ContentGetDefaultSortOrder;

begin

end.
